@@ -21,36 +21,42 @@
 #include <utility>
 #include <RcppArmadillo.h>
 #include "Abclass.h"
+#include "Control.h"
 #include "utils.h"
 
 namespace abclass
 {
-
     // the angle-based classifier with elastic-net penalty
     // estimation by coordinate-majorization-descent algorithm
-    class AbclassNet : public Abclass
+    template <typename T_loss, typename T_x>
+    class AbclassNet : public Abclass<T_loss, T_x>
     {
     protected:
+        using Abclass<T_loss, T_x>::inter_;
+        using Abclass<T_loss, T_x>::km1_;
+        using Abclass<T_loss, T_x>::mm_lowerbound_;
+        using Abclass<T_loss, T_x>::mm_lowerbound0_;
 
-        // for regularized coordinate majorization descent
-        arma::rowvec cmd_lowerbound_; // 1 by p1_
+        using Abclass<T_loss, T_x>::rescale_coef;
+        using Abclass<T_loss, T_x>::get_vertex_y;
+        using Abclass<T_loss, T_x>::objective0;
+        using Abclass<T_loss, T_x>::loss_derivative;
 
-        // pure virtual functions
-        virtual void set_cmd_lowerbound() = 0;
-        virtual double objective0(const arma::vec& inner) const = 0;
+        // cache
+        unsigned int num_iter_;   // number of CMD cycles till convergence
 
         // common methods
         inline double regularization(const arma::mat& beta,
                                      const double l1_lambda,
                                      const double l2_lambda) const
         {
-            if (intercept_) {
+            if (control_.intercept_) {
                 arma::mat beta0int { beta.tail_rows(p0_) };
                 return l1_lambda * l1_norm(beta0int) +
-                    l2_lambda * l2_norm_square(beta0int);
+                    0.5 * l2_lambda * l2_norm_square(beta0int);
             }
             return l1_lambda * l1_norm(beta) +
-                l2_lambda * l2_norm_square(beta);
+                0.5 * l2_lambda * l2_norm_square(beta);
         }
 
         // objective function with regularization
@@ -64,21 +70,24 @@ namespace abclass
         }
 
         // define gradient function at (l, j) for the given inner product
-        inline double cmd_gradient(const arma::vec& inner,
-                                   const arma::vec& vj_xl) const
+        inline double mm_gradient(const arma::vec& inner,
+                                  const arma::vec& vj_xl) const
         {
             arma::vec inner_grad { loss_derivative(inner) };
-            return arma::mean(obs_weight_ % vj_xl % inner_grad);
+            return arma::mean(control_.obs_weight_ % vj_xl % inner_grad);
         }
 
-        // gradient matrix
+        // gradient matrix regarding coef only (excluding intercept)
+        // only use it to determine the large-enough lambda
         inline arma::mat gradient(const arma::vec& inner) const
         {
-            arma::mat out { arma::zeros(p1_, km1_) };
+            arma::mat out { arma::zeros(p0_, km1_) };
             arma::vec inner_grad { loss_derivative(inner) };
             for (size_t j {0}; j < km1_; ++j) {
-                const arma::vec w_v_j { obs_weight_ % get_vertex_y(j) };
-                for (size_t l {0}; l < p1_; ++l) {
+                const arma::vec w_v_j {
+                    control_.obs_weight_ % get_vertex_y(j)
+                };
+                for (size_t l {0}; l < p0_; ++l) {
                     const arma::vec w_vj_xl { w_v_j % x_.col(l) };
                     out(l, j) = arma::mean(w_vj_xl % inner_grad);
                 }
@@ -86,24 +95,6 @@ namespace abclass
             return out;
         }
 
-        inline double max_diff(const arma::mat& beta_new,
-                               const arma::mat& beta_old) const
-        {
-            double out { 0.0 };
-            for (size_t j {0}; j < km1_; ++j) {
-                for (size_t i {0}; i < p1_; ++i) {
-                    double tmp {
-                        cmd_lowerbound_(i) *
-                        std::pow(beta_new(i, j) - beta_old(i, j), 2)
-                    };
-                    if (out < tmp) {
-                        out = tmp;
-                    }
-                }
-            }
-            return out;
-        }
-
         // run one cycle of coordinate descent over a given active set
         inline void run_one_active_cycle(arma::mat& beta,
                                          arma::vec& inner,
@@ -143,68 +134,32 @@ namespace abclass
     public:
 
         // inherit constructors
-        using Abclass::Abclass;
+        using Abclass<T_loss, T_x>::Abclass;
+        // specifics for template inheritance
+        using Abclass<T_loss, T_x>::control_;
+        using Abclass<T_loss, T_x>::x_;
+        using Abclass<T_loss, T_x>::p0_;
+        using Abclass<T_loss, T_x>::p1_;
+        using Abclass<T_loss, T_x>::n_obs_;
+        using Abclass<T_loss, T_x>::et_npermuted_;
+        using Abclass<T_loss, T_x>::coef_;
+        using Abclass<T_loss, T_x>::set_mm_lowerbound;
 
         // regularization
         // the "big" enough lambda => zero coef unless alpha = 0
         double l1_lambda_max_;
         double lambda_max_;
-        double alpha_;            // [0, 1]
-        arma::vec lambda_;        // lambda sequence
         // did user specified a customized lambda sequence?
         bool custom_lambda_ = false;
-        double lambda_min_ratio_ = 0.01;
-
-        // estimates
-        arma::cube coef_;         // p1_ by km1_
-
-        // tuning by cross-validation
-        arma::mat cv_accuracy_;
-        arma::vec cv_accuracy_mean_;
-        arma::vec cv_accuracy_sd_;
-
-        // control
-        double epsilon_;          // relative tolerance for convergence check
-        unsigned int max_iter_;   // maximum number of iterations
-        bool varying_active_set_; // if active set should be adaptive
-
-        // cache
-        unsigned int num_iter_;   // number of CMD cycles till convergence
 
         // for a sequence of lambda's
-        inline void fit(const arma::vec& lambda,
-                        const double alpha,
-                        const unsigned int nlambda,
-                        const double lambda_min_ratio,
-                        const unsigned int max_iter,
-                        const double epsilon,
-                        const bool varying_active_set,
-                        const unsigned int verbose);
-
-        // class conditional probability
-        inline arma::mat predict_prob(const arma::mat& beta,
-                                      const arma::mat& x) const
-        {
-            return Abclass::predict_prob(x * beta);
-        }
-        // prediction based on the inner products
-        inline arma::uvec predict_y(const arma::mat& beta,
-                                    const arma::mat& x) const
-        {
-            return Abclass::predict_y(x * beta);
-        }
-        // accuracy for tuning
-        inline double accuracy(const arma::mat& beta,
-                               const arma::mat& x,
-                               const arma::uvec& y) const
-        {
-            return Abclass::accuracy(x * beta, y);
-        }
+        inline void fit();
 
     };
 
     // run one CMD cycle over active sets
-    inline void AbclassNet::run_one_active_cycle(
+    template <typename T_loss, typename T_x>
+    inline void AbclassNet<T_loss, T_x>::run_one_active_cycle(
         arma::mat& beta,
         arma::vec& inner,
         arma::umat& is_active,
@@ -226,33 +181,37 @@ namespace abclass
         }
         for (size_t j {0}; j < km1_; ++j) {
             arma::vec v_j { get_vertex_y(j) };
-            for (size_t l {0}; l < p1_; ++l) {
+            // intercept
+            if (control_.intercept_) {
+                double dlj { mm_gradient(inner, v_j) };
+                double tmp_delta { - dlj / mm_lowerbound0_ };
+                beta(0, j) += tmp_delta;
+                inner += tmp_delta * v_j;
+            }
+            for (size_t l { 0 }; l < p0_; ++l) {
                 if (is_active(l, j) == 0) {
                     continue;
                 }
+                size_t l1 { l + inter_ };
                 arma::vec vj_xl { x_.col(l) % v_j };
-                double dlj { cmd_gradient(inner, vj_xl) };
-                double tmp { beta(l, j) };
-                // if cmd_lowerbound = 0 and l1_lambda > 0, numer will be 0
+                double dlj { mm_gradient(inner, vj_xl) };
+                double tmp { beta(l1, j) };
+                // if mm_lowerbound = 0 and l1_lambda > 0, numer will be 0
                 double numer {
-                    soft_threshold(
-                        cmd_lowerbound_(l) * beta(l, j) - dlj,
-                        l1_lambda * static_cast<double>(l >= int_intercept_)
-                        )
+                    soft_threshold(mm_lowerbound_(l) * beta(l1, j) - dlj,
+                                   l1_lambda)
                 };
                 // update beta
                 if (isAlmostEqual(numer, 0)) {
-                    beta(l, j) = 0;
+                    beta(l1, j) = 0;
                 } else {
-                    double denom { cmd_lowerbound_(l) + 2 * l2_lambda *
-                        static_cast<double>(l >= int_intercept_)
-                    };
-                    beta(l, j) = numer / denom;
+                    double denom { mm_lowerbound_(l) + l2_lambda };
+                    beta(l1, j) = numer / denom;
                 }
-                inner += (beta(l, j) - tmp) * vj_xl;
+                inner += (beta(l1, j) - tmp) * vj_xl;
                 if (update_active) {
                     // check if it has been shrinkaged to zero
-                    if (isAlmostEqual(beta(l, j), 0)) {
+                    if (isAlmostEqual(beta(l1, j), 0)) {
                         is_active(l, j) = 0;
                     } else {
                         is_active(l, j) = 1;
@@ -274,7 +233,8 @@ namespace abclass
     }
 
     // run CMD cycles over active sets
-    inline void AbclassNet::run_cmd_active_cycle(
+    template <typename T_loss, typename T_x>
+    inline void AbclassNet<T_loss, T_x>::run_cmd_active_cycle(
         arma::mat& beta,
         arma::vec& inner,
         arma::umat& is_active,
@@ -287,12 +247,13 @@ namespace abclass
         )
     {
         size_t i {0};
-        arma::mat beta0 { beta };
+        // arma::mat beta0 { beta };
+        double loss0 { objective0(inner) }, loss1 { loss0 };
         // use active-set if p > n ("helps when p >> n")
         if (varying_active_set) {
             arma::umat is_active_strong { is_active },
                 is_active_varying { is_active };
-            if (verbose > 1) {
+            if (verbose > 0) {
                 Rcpp::Rcout << "The size of active set from strong rule: "
                             << l1_norm(is_active_strong)
                             << "\n";
@@ -301,22 +262,29 @@ namespace abclass
                 // cycles over the active set
                 size_t ii {0};
                 while (ii < max_iter) {
+                    num_iter_ = ii + 1;
+                    Rcpp::checkUserInterrupt();
                     run_one_active_cycle(beta, inner, is_active_varying,
                                          l1_lambda, l2_lambda, true, verbose);
-                    if (rel_diff(beta0, beta) < epsilon) {
-                        num_iter_ = ii + 1;
+                    // if (rel_diff(beta0, beta) < epsilon) {
+                    //     break;
+                    // }
+                    // beta0 = beta;
+                    loss1 = objective0(inner);
+                    if (std::abs(loss1 - loss0) < epsilon) {
                         break;
                     }
-                    beta0 = beta;
+                    loss0 = loss1;
                     ii++;
                 }
                 // run a full cycle over the converged beta
                 run_one_active_cycle(beta, inner, is_active,
                                      l1_lambda, l2_lambda, true, verbose);
+                ++num_iter_;
                 // check two active sets coincide
                 if (l1_norm(is_active_varying - is_active) > 0) {
                     // if different, repeat this process
-                    if (verbose > 1) {
+                    if (verbose > 0) {
                         Rcpp::Rcout << "Changed the active set from "
                                     << l1_norm(is_active_varying)
                                     << " to "
@@ -330,41 +298,52 @@ namespace abclass
                     is_active = is_active_strong;
                     i++;
                 } else {
-                    if (verbose > 1) {
+                    if (verbose > 0) {
                         Rcpp::Rcout << "Converged over the active set after "
-                                    << num_iter_ + 1
+                                    << num_iter_
                                     << " iteration(s)\n";
+                        Rcpp::Rcout << "The size of active set is "
+                                    << l1_norm(is_active) << "\n";
                     }
-                    num_iter_ = i + 1;
                     break;
                 }
+                if (verbose > 0) {
+                    msg("Outer loop reached the maximum number of iteratons.");
+                }
             }
         } else {
             // regular coordinate descent
             while (i < max_iter) {
+                Rcpp::checkUserInterrupt();
+                num_iter_ = i + 1;
                 run_one_active_cycle(beta, inner, is_active,
                                      l1_lambda, l2_lambda, false, verbose);
-                if (rel_diff(beta0, beta) < epsilon) {
-                    num_iter_ = i + 1;
+                // if (rel_diff(beta0, beta) < epsilon) {
+                //     break;
+                // }
+                // beta0 = beta;
+                loss1 = objective0(inner);
+                if (std::abs(loss1 - loss0) < epsilon) {
                     break;
                 }
-                beta0 = beta;
+                loss0 = loss1;
                 i++;
             }
-        }
-        if (verbose > 0) {
-            if (num_iter_ < max_iter) {
-                Rcpp::Rcout << "Converged after "
-                            << num_iter_
-                            << " iteration(s)\n";
-            } else {
-                msg("Reached the maximum number of iteratons.");
+            if (verbose > 0) {
+                if (num_iter_ < max_iter) {
+                    Rcpp::Rcout << "Outer loop converged after "
+                                << num_iter_
+                                << " iteration(s)\n";
+                } else {
+                    msg("Outer loop reached the maximum number of iteratons.");
+                }
             }
         }
     }
 
     // one full cycle for coordinate-descent
-    inline void AbclassNet::run_one_full_cycle(
+    template <typename T_loss, typename T_x>
+    inline void AbclassNet<T_loss, T_x>::run_one_full_cycle(
         arma::mat& beta,
         arma::vec& inner,
         const double l1_lambda,
@@ -382,28 +361,32 @@ namespace abclass
         }
         for (size_t j {0}; j < km1_; ++j) {
             arma::vec v_j { get_vertex_y(j) };
-            for (size_t l {0}; l < p1_; ++l) {
+            // intercept
+            if (control_.intercept_) {
+                double dlj { mm_gradient(inner, v_j) };
+                double tmp_delta { - dlj / mm_lowerbound0_ };
+                beta(0, j) += tmp_delta;
+                inner += tmp_delta * v_j;
+            }
+            // others
+            for (size_t l { 0 }; l < p0_; ++l) {
+                size_t l1 { l + inter_ };
                 arma::vec vj_xl { v_j % x_.col(l) };
-                double dlj { cmd_gradient(inner, vj_xl) };
-                double tmp { beta(l, j) };
+                double dlj { mm_gradient(inner, vj_xl) };
+                double tmp { beta(l1, j) };
                 // if cmd_lowerbound = 0 and l1_lambda > 0, numer will be 0
                 double numer {
-                    soft_threshold(
-                        cmd_lowerbound_(l) * tmp - dlj,
-                        l1_lambda * static_cast<double>(l >= int_intercept_)
-                        )
+                    soft_threshold(mm_lowerbound_(l) * tmp - dlj,
+                                   l1_lambda)
                 };
                 // update beta
                 if (isAlmostEqual(numer, 0)) {
-                    beta(l, j) = 0;
+                    beta(l1, j) = 0;
                 } else {
-                    double denom {
-                        cmd_lowerbound_(l) + 2 * l2_lambda *
-                        static_cast<double>(l >= int_intercept_)
-                    };
-                    beta(l, j) = numer / denom;
+                    double denom { mm_lowerbound_(l) + l2_lambda };
+                    beta(l1, j) = numer / denom;
                 }
-                inner += (beta(l, j) - tmp) * vj_xl;
+                inner += (beta(l1, j) - tmp) * vj_xl;
             }
         }
         if (verbose > 1) {
@@ -420,7 +403,8 @@ namespace abclass
     }
 
     // run full cycles till convergence or reach max number of iterations
-    inline void AbclassNet::run_cmd_full_cycle(
+    template <typename T_loss, typename T_x>
+    inline void AbclassNet<T_loss, T_x>::run_cmd_full_cycle(
         arma::mat& beta,
         arma::vec& inner,
         const double l1_lambda,
@@ -430,14 +414,21 @@ namespace abclass
         const unsigned int verbose
         )
     {
-        arma::mat beta0 { beta };
+        // arma::mat beta0 { beta };
+        double loss0 { objective0(inner) }, loss1 { loss0 };
         for (size_t i {0}; i < max_iter; ++i) {
+            Rcpp::checkUserInterrupt();
+            num_iter_ = i + 1;
             run_one_full_cycle(beta, inner, l1_lambda, l2_lambda, verbose);
-            if (rel_diff(beta0, beta) < epsilon) {
-                num_iter_ = i + 1;
+            // if (rel_diff(beta0, beta) < epsilon) {
+            //     break;
+            // }
+            // beta0 = beta;
+            loss1 = objective0(inner);
+            if (std::abs(loss1 - loss0) < epsilon) {
                 break;
             }
-            beta0 = beta;
+            loss0 = loss1;
         }
         if (verbose > 0) {
             if (num_iter_ < max_iter) {
@@ -452,95 +443,97 @@ namespace abclass
 
     // for a sequence of lambda's
     // lambda * (alpha * lasso + (1 - alpha) / 2 * ridge)
-    inline void AbclassNet::fit(
-        const arma::vec& lambda,
-        const double alpha,
-        const unsigned int nlambda,
-        const double lambda_min_ratio,
-        const unsigned int max_iter,
-        const double epsilon,
-        const bool varying_active_set,
-        const unsigned int verbose
-        )
+    template <typename T_loss, typename T_x>
+    inline void AbclassNet<T_loss, T_x>::fit()
     {
         // set the CMD lowerbound
-        set_cmd_lowerbound();
-        // check alpha
-        if ((alpha < 0) || (alpha > 1)) {
-            throw std::range_error("The 'alpha' must be between 0 and 1.");
-        }
-        alpha_ = alpha;
-        // record control
-        epsilon_ = epsilon;
-        max_iter_ = max_iter;
-        varying_active_set_ = varying_active_set;
+        set_mm_lowerbound();
+        // TODO: allow groupwise adaptive weights
         // initialize
         arma::vec one_inner { arma::zeros(n_obs_) };
         arma::mat one_beta { arma::zeros(p1_, km1_) },
             one_grad_beta { one_beta };
-        const bool is_ridge_only { isAlmostEqual(alpha, 0.0) };
-        double l1_lambda, l2_lambda;
-        // if alpha = 0 and lambda is specified
-        if (is_ridge_only && ! lambda.empty()) {
-            lambda_ = arma::reverse(arma::unique(lambda));
+        const bool is_ridge_only { isAlmostEqual(control_.alpha_, 0.0) };
+        double l1_lambda { 0.0 }, l2_lambda { 0.0 };
+        if (! control_.lambda_.empty()) {
+            control_.lambda_ = arma::reverse(arma::unique(control_.lambda_));
+            control_.nlambda_ = control_.lambda_.n_elem;
             custom_lambda_ = true;
+        }
+        // if alpha = 0 and customized lambda
+        if (is_ridge_only && custom_lambda_) {
             l1_lambda_max_ = - 1.0; // not well defined
             lambda_max_ = - 1.0;    // not well defined
         } else {
             // need to determine lambda_max
             one_grad_beta = arma::abs(gradient(one_inner));
-            // large enough lambda for all-zero coef (except intercept terms)
-            l1_lambda_max_ = one_grad_beta.tail_rows(p0_).max();
-            lambda_max_ =  l1_lambda_max_ / std::max(alpha, 1e-2);
+            l1_lambda_max_ = one_grad_beta.max();
+            lambda_max_ =  l1_lambda_max_ / std::max(control_.alpha_, 1e-2);
             // set up lambda sequence
-            if (lambda.empty()) {
+            if (! custom_lambda_) {
                 double log_lambda_max { std::log(lambda_max_) };
-                lambda_ = arma::exp(
+                control_.lambda_ = arma::exp(
                     arma::linspace(log_lambda_max,
-                                   log_lambda_max + std::log(lambda_min_ratio),
-                                   nlambda)
+                                   log_lambda_max +
+                                   std::log(control_.lambda_min_ratio_),
+                                   control_.nlambda_)
                     );
-                lambda_min_ratio_ = lambda_min_ratio;
-            } else {
-                lambda_ = arma::reverse(arma::unique(lambda));
-                custom_lambda_ = true;
             }
         }
         // initialize the estimate cube
-        coef_ = arma::cube(p1_, km1_, lambda_.n_elem);
+        coef_ = arma::cube(p1_, km1_, control_.lambda_.n_elem,
+                           arma::fill::zeros);
+        this->loss_wo_penalty_ = arma::zeros(control_.lambda_.n_elem);
+        this->penalty_ = this->loss_wo_penalty_;
         // for ridge penalty
         if (is_ridge_only) {
-            for (size_t li { 0 }; li < lambda_.n_elem; ++li) {
-                run_cmd_full_cycle(one_beta, one_inner,
-                                   0.0, 0.5 * lambda_(li),
-                                   max_iter, epsilon, verbose);
+            for (size_t li { 0 }; li < control_.lambda_.n_elem; ++li) {
+                l2_lambda = control_.lambda_(li);
+                run_cmd_full_cycle(one_beta,
+                                   one_inner,
+                                   l1_lambda,
+                                   l2_lambda,
+                                   control_.max_iter_,
+                                   control_.epsilon_,
+                                   control_.verbose_);
                 coef_.slice(li) = rescale_coef(one_beta);
+                this->loss_wo_penalty_(li) = objective0(one_inner);
+                this->penalty_(li) = regularization(
+                    one_beta, l1_lambda, l2_lambda);
             }
             return;             // early exit
         }
         // else, not just ridge penalty with l1_lambda > 0
         double one_strong_rhs { 0.0 };
-        l2_lambda = 0.5 * lambda_max_ * (1 - alpha);
+        l2_lambda = lambda_max_ * (1 - control_.alpha_);
         // get the solution (intercepts) of l1_lambda_max for a warm start
-        arma::umat is_active_strong { arma::zeros<arma::umat>(p1_, km1_) };
-        if (intercept_) {
+        arma::umat is_active_strong { arma::zeros<arma::umat>(p0_, km1_) };
+        if (control_.intercept_) {
             // only need to estimate intercept
-            is_active_strong.row(0) = arma::ones<arma::umat>(1, km1_);
-            run_cmd_active_cycle(one_beta, one_inner, is_active_strong,
-                                 l1_lambda_max_, l2_lambda,
-                                 false, max_iter, epsilon, verbose);
+            run_cmd_active_cycle(one_beta,
+                                 one_inner,
+                                 is_active_strong,
+                                 l1_lambda_max_,
+                                 l2_lambda,
+                                 false,
+                                 control_.max_iter_,
+                                 control_.epsilon_,
+                                 control_.verbose_);
         }
         // optim with varying active set when p > n
         double old_l1_lambda { l1_lambda_max_ }; // for strong rule
         // main loop: for each lambda
-        for (size_t li { 0 }; li < lambda_.n_elem; ++li) {
-            double lambda_li { lambda_(li) };
-            l1_lambda = lambda_li * alpha;
-            l2_lambda = 0.5 * lambda_li * (1 - alpha);
+        for (size_t li { 0 }; li < control_.lambda_.n_elem; ++li) {
+            double lambda_li { control_.lambda_(li) };
+            l1_lambda = lambda_li * control_.alpha_;
+            l2_lambda = lambda_li * (1 - control_.alpha_);
             // early exit for lambda greater than lambda_max_
             // note that lambda is sorted
             if (l1_lambda >= l1_lambda_max_) {
                 coef_.slice(li) = rescale_coef(one_beta);
+                this->loss_wo_penalty_(li) = objective0(one_inner);
+                this->penalty_(li) = regularization(
+                    one_beta, l1_lambda, l2_lambda);
                 continue;
             }
             // update active set by strong rule
@@ -548,7 +541,7 @@ namespace abclass
             one_strong_rhs = 2 * l1_lambda - old_l1_lambda;
             old_l1_lambda = l1_lambda;
             for (size_t j { 0 }; j < km1_; ++j) {
-                for (size_t l { int_intercept_ }; l < p1_; ++l) {
+                for (size_t l { 0 }; l < p0_; ++l) {
                     if (is_active_strong(l, j) > 0) {
                         continue;
                     }
@@ -566,21 +559,27 @@ namespace abclass
                     arma::zeros<arma::umat>(arma::size(is_active_strong))
                 };
                 // update beta
-                run_cmd_active_cycle(one_beta, one_inner, is_active_strong,
-                                     l1_lambda, l2_lambda, varying_active_set,
-                                     max_iter, epsilon, verbose);
-                if (verbose > 0) {
+                run_cmd_active_cycle(one_beta,
+                                     one_inner,
+                                     is_active_strong,
+                                     l1_lambda,
+                                     l2_lambda,
+                                     control_.varying_active_set_,
+                                     control_.max_iter_,
+                                     control_.epsilon_,
+                                     control_.verbose_);
+                if (control_.verbose_ > 0) {
                     msg("Checking the KKT condition for the null set.");
                 }
                 // check kkt condition
                 for (size_t j { 0 }; j < km1_; ++j) {
                     arma::vec v_j { get_vertex_y(j) };
-                    for (size_t l { int_intercept_ }; l < p1_; ++l) {
+                    for (size_t l { 0 }; l < p0_; ++l) {
                         if (is_active_strong_old(l, j) > 0) {
                             continue;
                         }
                         arma::vec vj_xl { v_j % x_.col(l) };
-                        if (std::abs(cmd_gradient(one_inner, vj_xl)) >
+                        if (std::abs(mm_gradient(one_inner, vj_xl)) >
                             one_strong_rhs) {
                             // update active set
                             is_strong_rule_failed(l, j) = 1;
@@ -590,7 +589,7 @@ namespace abclass
                 if (arma::accu(is_strong_rule_failed) > 0) {
                     is_active_strong = is_active_strong_old ||
                         is_strong_rule_failed;
-                    if (verbose > 0) {
+                    if (control_.verbose_ > 0) {
                         Rcpp::Rcout << "The strong rule failed.\n"
                                     << "The size of old active set: "
                                     << l1_norm(is_active_strong_old)
@@ -599,13 +598,47 @@ namespace abclass
                                     << "\n";
                     }
                 } else {
-                    if (verbose > 0) {
+                    if (control_.verbose_ > 0) {
                         msg("The strong rule worked.\n");
                     }
                     kkt_failed = false;
                 }
             }
+            if (et_npermuted_ > 0) {
+                if (control_.verbose_ > 0) {
+                    msg("[ET] check if any pseudo-predictors was selected.");
+                }
+                // assume the last (permuted ) predictors are inactive
+                arma::mat permuted_beta { one_beta.tail_rows(et_npermuted_) };
+                if (! permuted_beta.is_zero(arma::datum::eps)) {
+                    if (li == 0) {
+                        msg("Warning: Fail to tune by ET-lasso; ",
+                            "selected pseudo-predictor(s) by ",
+                            "the largest lamabda.\n",
+                            "Suggestion: increase 'lambda', ",
+                            "'lambda_min_ratio' or 'nlambda'?");
+                    } else {
+                        coef_ = coef_.head_slices(li);
+                    }
+                    if (control_.verbose_ > 0) {
+                        msg("[ET] selected pseudo-predictor(s).\n");
+                    }
+                    break;
+                }
+                if (control_.verbose_ > 0) {
+                    msg("[ET] none of pseudo-predictors was selected.\n");
+                }
+                if (li == control_.lambda_.n_elem - 1) {
+                    msg("Warning: Fail to tune by ET-lasso; ",
+                        "no pseudo-predictors selected ",
+                        "by the smallest lambda.\n",
+                        "Suggestion: decrease 'lambda' or 'lambda_min_ratio'?");
+                }
+            }
             coef_.slice(li) = rescale_coef(one_beta);
+            this->loss_wo_penalty_(li) = objective0(one_inner);
+            this->penalty_(li) = regularization(
+                one_beta, l1_lambda, l2_lambda);
         }
     }
 
